#module  CC_Parse_DCL  "V1.0"

/*++CCPARSE.C

 Facility: 
	Fermilab Accelerator Control System - VAX/VMS ACNET
	General utility routine for VAX C.

 Abstract:
	This module contains a routine to interface a VAX C program to the
	CLI$DCL_PARSE routine where the VAX C program is activated as a
	foreign command and the DCL syntax tables are integrated into the
	executable image.

 Environment:
	User mode, usually called from main().
--*/

/*
 Author: Frank J. Nagy	    Fermilab Accelerator Controls
 Modification History:

 V1.0	22-Feb-86  FJN	Created for V2.0 of GetQuota
*/

/******************
 * Include Files: *
 ******************/
#include  vaxtypes			/* VAX-11 C Type extensions */
#include  descrip			/* VAX-11 Descriptors */
#include  descrip2			/* Additional descriptor defs. */
#include  stsdef			/* Fields of status code */

/********************************
 * Local definitions and types: *
 ********************************/
typedef unsigned long status;		/* Condition code status type */

#define FOREIGN_LEN	255		/* Foreign command line length */
#define	VERB_LEN	31		/* Maximum DCL verb size for here */
#define	BUF_LEN	    (VERB_LEN + FOREIGN_LEN)

/************************
 * Local (own) storage: *
 ************************/
static char line_buffer[BUF_LEN];	/* Command line buffer */

/********************************
 * Global procedures/functions: *
 ********************************/
status LIB$GET_FOREIGN();		/* Get foreign command line */
status CLI$DCL_PARSE();			/* Parse DCL command line */
status LIB$GET_INPUT();			/* Get input from SYS$INPUT */

int strlen();				/* Length of NUL-terminated string */

/*+ CC_PARSE_DCL
 Interface for VAX C to CLI$DCL_PARSE to convert from foreign command to
 DCL and parse with the provided DCL tables.

      CC_PARSE_DCL( verb, tables)

  verb     character string giving the DCL verb for the command defined
           in the DCL syntax tables.

  tables   pointer to the DCL syntax tables.
-*/

CC_PARSE_DCL( verb, tables)
char *verb;				/* Command verb text */
char *tables;				/* Pointer to DCL tables */
{
status sts;
int n;
$DESCRIPTOR( d_line, line_buffer);	/* Descriptor for DCL command line */
$DESCRIPTOR0( d_foreign);		/* Descriptor for foreign command */

n = strlen( verb);			/* Length of DCL verb */
if (n > VERB_LEN)
    n = VERB_LEN;

strncpy( line_buffer, verb, n);		/* Copy verb to buffer */

d_foreign.dsc$w_length = FOREIGN_LEN;	/* Setup foreign command descriptor */
d_foreign.dsc$a_pointer = d_line.dsc$a_pointer + n;
*(d_foreign.dsc$a_pointer)++ = ' ';	/* Separate verb and parameters! */

sts = LIB$GET_FOREIGN( &d_foreign, 0, &d_foreign.dsc$w_length);
SIGNAL_FAILURE( sts);

d_line.dsc$w_length = d_foreign.dsc$w_length + n + 1;

sts = CLI$DCL_PARSE( &d_line, tables, LIB$GET_INPUT);
SIGNAL_FAILURE( sts);

}
