/*+ $UAFDEF
   Module ($UAFDEF) of FERMI$LIB:LCLCDEF.TLB.
 
  Defines the format of the VMS V4 User Authorization File record as
  the structure UAF_RECORD.
-*/
/* Extracted $UAFDEF from SYS$LIBRARY:LIB on 03-Jun-85 and editted manually */
/* Editted for V4.4 on 28-Jun-86. */

#define  UAF$S_UAFDEF		1412	/* Maximum record size */
					/* Size of ... field */
#define  UAF$S_USERNAME		32	/* username text */
#define  UAF$S_PARENT_ID	8
#define  UAF$S_ACCOUNT		32	/* account name text */
#define  UAF$S_OWNER		32	/* owner's name text */
#define  UAF$S_DEFDEV		32	/* default device text */
#define  UAF$S_DEFDIR		64	/* default directory text */
#define  UAF$S_LGICMD		64	/* default login command file */
#define  UAF$S_DEFCLI		32	/* command interpreter name */
#define  UAF$S_CLITABLES	32	/* CLI tables file name */
#define  UAF$S_PWD		8
#define  UAF$S_PWD2		8
#define  UAF$S_EXPIRATION	8
#define  UAF$S_PWD_LIFETIME	8
#define  UAF$S_PWD_DATE		8
#define  UAF$S_PWD2_DATE	8
#define  UAF$S_LASTLOGIN_I	8
#define  UAF$S_LASTLOGIN_N	8
#define  UAF$S_PRIV		8
#define  UAF$S_DEF_PRIV		8
#define  UAF$S_MIN_CLASS	20	/* Minimum security class */
#define  UAF$S_MAX_CLASS	20	/* Maximum security class */
					/* Access hours (0..23): */
#define  UAF$S_NETWORK_ACCESS_P	3
#define  UAF$S_NETWORK_ACCESS_S	3
#define  UAF$S_BATCH_ACCESS_P	3
#define  UAF$S_BATCH_ACCESS_S	3
#define  UAF$S_LOCAL_ACCESS_P	3
#define  UAF$S_LOCAL_ACCESS_S	3
#define  UAF$S_DIALUP_ACCESS_P	3
#define  UAF$S_DIALUP_ACCESS_S	3
#define  UAF$S_REMOTE_ACCESS_P	3
#define  UAF$S_REMOTE_ACCESS_S	3
					/* All text fields are blank padded
					   and, except for username and account,
					   are counted. */

#define  UAF$C_USER_ID		1	/* Main user ID record */
#define  UAF$C_VERSION1		1	/* VMS V4 UAF format */
#define  UAF$C_KEYED_PART	52	/* ISAM keys in this part... */

struct	UAF_RECORD {
	unsigned char uaf$b_rtype;	/* Record type code */
	unsigned char uaf$b_version;	/* UAF format version */
	unsigned short uaf$w_usrdatoff;	/* Offset to counted string user data */
					/* Username text field (key #0): */
	char uaf$t_username[UAF$S_USERNAME];
#define	uaf$t_username_tag	uaf$t_username[UAF$S_USERNAME-1]
	variant_union {
	    unsigned long uaf$l_uic;	/* User Id Code (UIC) field (key #1) */
	    variant_struct {
		unsigned short uaf$w_mem;   /* Member number word of UIC (key #2) */
		unsigned short uaf$w_grp;   /* Group number word of UIC */
		} uaf$r_uic_fields;
	    } uaf$r_uic_overlay;
	unsigned long uaf$l_sub_id;	/* User sub-identifer */
					/* Identifier of owner of this account */
	unsigned long uaf$q_parent_id[2];
					/* Account name field: */
	char uaf$t_account[UAF$S_ACCOUNT];
	char uaf$t_owner[UAF$S_OWNER];	/* Owner's name text */
					/* Default device counted string: */
	char uaf$t_defdev[UAF$S_DEFDEV];
					/* Default directory counted string: */
	char uaf$t_defdir[UAF$S_DEFDIR];
					/* Login command file counted string: */
	char uaf$t_lgicmd[UAF$S_LGICMD];
					/* Default command interpreter: */
	char uaf$t_defcli[UAF$S_DEFCLI];
					/* Default command tables: */
	char uaf$t_clitables[UAF$S_CLITABLES];
	variant_union {
	    unsigned long uaf$q_pwd[2];	/* Hashed password */
	    unsigned long uaf$l_pwd;	/* Hashed password subfield */
	    } uaf$r_pwd_overlay;
	unsigned long uaf$q_pwd2[2];	/* Hashed secondary password */
	unsigned short uaf$w_logfails;	/* Count of login failures */
	unsigned short uaf$w_salt;	/* Random password salt */
	char uaf$b_encrypt;		/* Encryption algorithm code */
	char uaf$b_encrypt2;		/* Secondary password algorithm code */
#define  UAF$C_AD_II	0	/* Autodin-II 32 bit CRC encryption code */
#define  UAF$C_PURDY	1	/* Purdy Polynomial over salted input */
#define  UAF$C_PURDY_V	2	/* Purdy Polynomial + variable length username */
	unsigned char uaf$b_pwd_length;	/* Minimum password length */
	char _fill_1;
					/* Dates/lifetimes: */
	unsigned long uaf$q_expiration[2];	/* Account expiration date */
	unsigned long uaf$q_pwd_lifetime[2];	/* Password lifetime */
	unsigned long uaf$q_pwd_date[2];	/* Password changed date */
	unsigned long uaf$q_pwd2_date[2];	/* 2nd password changed */
	unsigned long uaf$q_lastlogin_i[2];	/* Last interactive login */
	unsigned long uaf$q_lastlogin_n[2];	/* Last non-interactive login */
	unsigned long uaf$q_priv[2];	/* Authorized process privileges */
	unsigned long uaf$q_def_priv[2]; /* Default processes privileges */
	char uaf$r_min_class[UAF$S_MIN_CLASS];	/* Minimum security class */
	char uaf$r_max_class[UAF$S_MAX_CLASS];	/* Maximum security class */
	variant_union {
	    unsigned long uaf$l_flags;	/* User flags */
	    variant_struct {
		int uaf$v_disctly : 1;		/* Don't allow user control-Y */
		int uaf$v_defcli : 1;		/* Only allow user default CLI */
		int uaf$v_lockpwd : 1;		/* Disable SET PASSWORD command */
		int uaf$v_captive : 1;		/* No overrides allowed on login */
		int uaf$v_disacnt : 1;		/* Disable account */
		int uaf$v_diswelcom  : 1;	/* Skip welcome message */
		int uaf$v_dismail : 1;		/* Skip new mail message */
		int uaf$v_nomail : 1;		/* Disable mail delivery */
		int uaf$v_genpwd : 1;		/* Passwords must be generated */
		int uaf$v_pwd_expired   : 1;	/* Password has expired */
		int uaf$v_pwd2_expired  : 1;	/* 2nd password has expired */
		int uaf$v_audit : 1;		/* Audit all actions */
		int uaf$v_disreport  : 1;	/* Skip last login message */
		int uaf$v_disreconnect  : 1;	/* Inhibit reconnections */
		int uaf$v_autologin : 1;	/* auto-login only */
		} uaf$r_flags_bits;
	    } uaf$r_flags_overlay;
				/* Hourly access restrictions for primary (_p)
				   and secondary (_s) days via: */
	unsigned char uaf$b_network_access_p[UAF$S_NETWORK_ACCESS_P];
	unsigned char uaf$b_network_access_s[UAF$S_NETWORK_ACCESS_S];
	unsigned char uaf$b_batch_access_p[UAF$S_BATCH_ACCESS_P];
	unsigned char uaf$b_batch_access_s[UAF$S_BATCH_ACCESS_S];
	unsigned char uaf$b_local_access_p[UAF$S_LOCAL_ACCESS_P];
	unsigned char uaf$b_local_access_s[UAF$S_LOCAL_ACCESS_S];
	unsigned char uaf$b_dialup_access_p[UAF$S_DIALUP_ACCESS_P];
	unsigned char uaf$b_dialup_access_s[UAF$S_DIALUP_ACCESS_S];
	unsigned char uaf$b_remote_access_p[UAF$S_REMOTE_ACCESS_P];
	unsigned char uaf$b_remote_access_s[UAF$S_REMOTE_ACCESS_S];
	unsigned char _fill_4[12];
	unsigned char uaf$b_primedays;	/* Bit representing primary days */
#define  UAF$V_MONDAY     0	/* 0=primary day, 1=secondary day */
#define  UAF$V_TUESDAY    1
#define  UAF$V_WEDNESDAY  2
#define  UAF$V_THURSDAY   3
#define  UAF$V_FRIDAY     4
#define  UAF$V_SATURDAY   5
#define  UAF$V_SUNDAY     6
#define  UAF$M_MONDAY		(1 << UAF$V_MONDAY)
#define  UAF$M_TUESDAY		(1 << UAF$V_TUESDAY)
#define  UAF$M_WEDNESDAY	(1 << UAF$V_WEDNESDAY)
#define  UAF$M_THURSDAY		(1 << UAF$V_THURSDAY)
#define  UAF$M_FRIDAY		(1 << UAF$V_FRIDAY)
#define  UAF$M_SATURDAY		(1 << UAF$V_SATURDAY)
#define  UAF$M_SUNDAY		(1 << UAF$V_SUNDAY)
	char _fill_5;
	unsigned char uaf$b_pri;	/* Base process priority */
	unsigned char uaf$b_quepri;	/* Maximum job queuing priority */
					/* 0=no limit for next 3 fields */
	unsigned short uaf$w_maxjobs;	/* Maximum jobs for UIC */
	unsigned short uaf$w_maxacctjobs; /* Maximum jobs for account */
	unsigned short uaf$w_maxdetach;	/* Maximum detached processes for UIC */
	unsigned short uaf$w_prccnt;	/* Subprocess creation limit */
	unsigned short uaf$w_biolm;	/* Buffered I/O limit */
	unsigned short uaf$w_diolm;	/* Direct I/O limit */
	unsigned short uaf$w_tqcnt;	/* Timer queue entry limit */
	unsigned short uaf$w_astlm;	/* AST queue limit */
	unsigned short uaf$w_enqlm;	/* Enqueue limit */
	unsigned short uaf$w_fillm;	/* Open file limit */
	unsigned short uaf$w_shrfillm;	/* Shared file limit */
	unsigned long uaf$l_wsquota;	/* Working set size quota */
	unsigned long uaf$l_dfwscnt;	/* Default working set size */
	unsigned long uaf$l_wsextent;	/* Working set size limt */
	unsigned long uaf$l_pgflquota;	/* Paging file quota */
	unsigned long uaf$l_cputim;	/* CPU time quota */
	unsigned long uaf$l_bytlm;	/* Buffered I/O byte count limit */
	unsigned long uaf$l_pbytlm;	/* Paged buffer I/O byte count limit */
	unsigned long uaf$l_jtquota;	/* Job-wide logical name table limit */
	unsigned short uaf$w_proxy_lim;	/* # of proxies user can grant */
	unsigned short uaf$w_proxies;	/* # of proxies granted */
	unsigned short uaf$w_account_lim; /* # of sub-accounts allowed */
	unsigned short uaf$w_accounts;	/* # of sub-accounts in use */
	char _fill_99[64];
	};

#define  UAF$V_DISCTLY	 0	/* Don't allow user control-Y */
#define  UAF$V_DEFCLI	 1	/* Only allow user default CLI */
#define  UAF$V_LOCKPWD	 2	/* Disable SET PASSWORD command */
#define  UAF$V_CAPTIVE	 3	/* No overrides allowed on login */
#define  UAF$V_DISACNT	 4	/* Disable account */
#define  UAF$V_DISWELCOM 5	/* Skip welcome message */
#define  UAF$V_DISMAIL	 6	/* Skip new mail message */
#define  UAF$V_NOMAIL	 7	/* Disable mail delivery */
#define  UAF$V_GENPWD	 8	/* Passwords must be generated */
#define  UAF$V_PWD_EXPIRED  9	/* Password has expired */
#define  UAF$V_PWD2_EXPIRED 10	/* 2nd password has expired */
#define  UAF$V_AUDIT	 11	/* Audit all actions */
#define  UAF$V_DISREPORT 12	/* Skip last login message */
#define  UAF$V_DISRECONNECT 13	/* Inhibit reconnections */
#define  UAF$V_AUTOLOGIN 14	/* auto-login only */
#define  UAF$M_DISCTLY		(1 << UAF$V_DISCTLY)
#define  UAF$M_DEFCLI		(1 << UAF$V_DEFCLI)
#define  UAF$M_LOCKPWD		(1 << UAF$V_LOCKPWD)
#define  UAF$M_CAPTIVE		(1 << UAF$V_CAPTIVE)
#define  UAF$M_DISACNT		(1 << UAF$V_DISACNT)
#define  UAF$M_DISWELCOM	(1 << UAF$V_DISWELCOM)
#define  UAF$M_DISMAIL		(1 << UAF$V_DISMAIL)
#define  UAF$M_NOMAIL		(1 << UAF$V_NOMAIL)
#define  UAF$M_GENPWD		(1 << UAF$V_GENPWD)
#define  UAF$M_PWD_EXPIRED	(1 << UAF$V_PWD_EXPIRED)
#define  UAF$M_PWD2_EXPIRED	(1 << UAF$V_PWD2_EXPIRED)
#define  UAF$M_AUDIT		(1 << UAF$V_AUDIT)
#define  UAF$M_DISREPORT	(1 << UAF$V_DISREPORT)
#define  UAF$M_DISRECONNECT	(1 << UAF$V_DISRECONNECT)
#define  UAF$M_AUTOLOGIN	(1 << UAF$V_AUTOLOGIN)

#define  UAF$C_FIXED      644	/* Size of fixed portion of record */
#define  UAF$C_LENGTH     1412	/* Maximum size of record */
#define  UAF$K_FIXED      644	/* Size of fixed portion of record */
#define  UAF$K_LENGTH     1412	/* Maximum size of record */
/*
  End of $UAFDEF module.
*/
