/*+ DESCRIP2
 Module (DESCRIP2) of SYS$LIBRARY:LCLCDEF.TLB and file DESCRIP2.H.

 This module defines some additions to the standard VAX-11 C include
 module (DESCRIP) found in SYS$LIBRARY:CSYSDEF.TLB.

 The following macros are defined:

  $DESCRIPTOR0(name)             empty string descriptor (length and
                                 pointer fields initialzed to 0).

  $DESCRIPTOR1(name,string)      string descriptor for 1 character
                                 or a whole structure.

  $DESCRIPTORM(name,string)      string descriptor for structure
                                 member or non-NUL terminated strings.

  $DESCRIPTORA(name,array,type)  array descriptor.

  $LENGTH(descriptor)            length field of descriptor.
  $POINTER(descriptor)           pointer field of descriptor.
-*/
/*
  V1.0	16-May-83	FJN	Created
*/

/*	DESCRIP2 - Extended VMS Descriptor Definitions 	*/

/* Invoke standard descriptor definitions if not already done so */

#ifndef $DESCRIPTOR
#include descrip
#endif

/* 	simple macro to construct an empty string descriptor 	*/

#define $DESCRIPTOR0(name) \
	struct dsc$descriptor_s name = { 0, DSC$K_DTYPE_T, DSC$K_CLASS_S, 0 }

/* 	simple macro to construct string descriptor for single
	characters or for complete structures:			*/

#define $DESCRIPTOR1(name,string) \
	struct dsc$descriptor_s name = { sizeof(string), DSC$K_DTYPE_T, DSC$K_CLASS_S, &string }

/* 	macro to construct string descriptor for structure
	members or non-NUL terminated strings:			*/

#define $DESCRIPTORM(name,string) \
	struct dsc$descriptor_s name = { sizeof(string), DSC$K_DTYPE_T, DSC$K_CLASS_S, string }

/* 	macro to construct array desciptors:			*/

#define $DESCRIPTORA(name,array,type) \
	struct dsc$descriptor_a name = { sizeof(type), DSC$K_DTYPE_L, DSC$K_CLASS_A, array, 0, 0, \
					{ 0, 0, 0, 0, 0 }, 1, sizeof(array) }

/*	macros to reference the length or pointer fields of
	descriptors						*/

#define $LENGTH(descriptor) descriptor.dsc$w_length
#define $POINTER(descriptor) descriptor.dsc$a_pointer

/*
 End of DESCRIP2 module
*/
