#module  XDFO  "V1.0"

/* XDFO.C

 Facility: 
    Fermilab EPICURE Control System
    General VMS Utilities

 Abstract:
    This module implements a program to prepare various types of definitions
    file to define symbols using as input an object file.  The name stands
    for "eXtract Definitions From Object".

 Environment:
    User mode, foreign DCL command.

 Author: Frank J. Nagy		Fermilab, Research Division Controls
 Modification History:

 V0.0	31-Aug-86  FJN	Created for making QVIMSG.OPT from QVI.OBJ from QVI.MSG
 V0.1	01-Sep-86  FJN	Began realistic implementation with CLD tables and the
			whole 12-yards ...
 V1.0	02-Sep-86  FJN	Completed pseudo-DCL implementation with multiple types
			of output definitions
*/

/******************
 * Include Files: *
 ******************/
#include  vaxtypes			/* VAX-11 C Type extensions */
#include  descrip			/* VAX-11 Descriptors */
#include  descrip2			/* Additional descriptor defs. */
#include  fscndef			/* Definitions for SYS$FILESCAN */
#include  stsdef			/* Status code fields */

#include  stdio				/* Standard I/O functions */
#include  errno				/* VAX C Error definitions */

#include  "vaxobj.h"			/* VAX Object Language definitions */

/**************************************
 * Global definitions and references: *
 **************************************/
globalvalue SS$_NORMAL;			/* Success status */
globalvalue CLI$_ABSENT;		/* Qualifier absent status */

globalvalue XDFO__INPFILERR, XDFO__OUTFILERR;	/* File error signals */
globalvalue XDFO__MACRONAM;		/* Macro name without "DEF" */
globalvalue XDFO__NODEFTYP;		/* No definitions type selected */

globalref char XDFO_TABLES;		/* DCL tables for XDFO "command" */

#define  RO$DESCRIPTOR(d,s) readonly static $DESCRIPTOR(d,s)
/********************************
 * Local definitions and types: *
 ********************************/
typedef unsigned long status;		/* Condition code status type */
typedef struct dsc$descriptor *Descrp;

struct Item {				/* Element of an item list */
    unsigned short buflen;		/* Item buffer length */
    unsigned short itmcod;		/* Item code */
    address bufferp;			/* Pointer to item's buffer */
    unsigned long *rtnlenp;		/* Pointer to return length */
    };

struct Item_2 {				/* Item of list for SYS$FILESCAN */
    unsigned short clen;		/* Returns component length */
    unsigned short itmcod;		/* Item code of component */
    address caddr;			/* Pointer to item in input string */
    };

struct EIname {				/* Exclude/Include name descriptor */
    struct EIname *next;		/* Linked list */
    struct dsc$descriptor desc;		/* Descriptor for name */
    };

#define	FILE_LEN    255			/* File specification length */

#define	NAME_LEN    31			/* Maximum length of symbol names */
#define	NAME_BUF    32			/* Length of symbol name buffers */

					/* Return byte offset of member */
#define	OFFSET_OF(Structure,Member) ((int) &(((struct Structure *)0)->Member))

/*****************************
 * Module Local Own Storage: *
 *****************************/

					/* Parameter and qualifier names */
RO$DESCRIPTOR( d_p1, "P1");
RO$DESCRIPTOR( d_output, "OUTPUT");

RO$DESCRIPTOR( d_allhex, "ALLHEX");
RO$DESCRIPTOR( d_exclude, "EXCLUDE");
RO$DESCRIPTOR( d_include, "INCLUDE");
RO$DESCRIPTOR( d_absolute, "ABSOLUTE");
RO$DESCRIPTOR( d_relocatable, "RELOCATABLE");
RO$DESCRIPTOR( d_weak, "WEAK");

					/* Type qualifiers and file types */
RO$DESCRIPTOR( d_cc, "CC");
readonly static char *dna_cc = "dna=.H";
RO$DESCRIPTOR( d_fortran, "FORTRAN");
readonly static char *dna_fortran = "dna=.FOR";
RO$DESCRIPTOR( d_macro, "MACRO");
readonly static char *dna_macro = "dna=.MAC";
RO$DESCRIPTOR( d_options, "OPTIONS");
readonly static char *dna_options = "dna=.OPT";
RO$DESCRIPTOR( d_pascal, "PASCAL");
readonly static char *dna_pascal = "dna=.PAS";

static char ftext[512] = "XDFO ";	/* Buffer for foreign command line */
static $DESCRIPTOR( d_ftext, ftext);
#define	VERB_LEN    (sizeof "XDFO " - 1)

static char objfile[255], deffile[255];	/* File specification buffers */
static $DESCRIPTOR( d_objfile, objfile);
static $DESCRIPTOR( d_deffile, deffile);

static FILE *obj, *defs;		/* Input and output file pointers */

static struct {
    struct EIname *first, *last;	/* Pointers to 1st/last list entries */
    } excludes = { NULL, NULL };	/* List of excluded names */

static struct {
    struct EIname *first, *last;	/* Pointers to 1st/last list entries */
    } includes = { NULL, NULL };	/* List of included names */

static int all_hex = FALSE;		/* Only mask and status values in hex */
static int use_absolute = TRUE;		/* Include ABSOLUTE symbols */
static int use_relocatable = FALSE;	/* Include RELOCATABLE symbols */
static int use_weak = TRUE;		/* Include WEAK symbols */

					/* Definitions output type selector */
static enum { CC, FOR, MAC, OPT, PAS } deftype;

static struct {
    struct Item_2 filename;		/* Return file name string */
    struct Item_2 filetype;		/* Return file type string */
    unsigned long listend;
    } fileparts =			/* For parsing file specifications */
	{ { 0, FSCN$_NAME, NULL }, { 0, FSCN$_TYPE, NULL }, 0 };

/*
 Character sequences to begin, continue and end comment blocks for the
 different types of definitions.
*/
					/*  CC	FOR MAC	OPT PAS */
readonly static char *begin_comment[] =	{ "/*", "C", ";", "!", "{" };
readonly static char *cont_comment[] =	{ "",	"C", ";", "!", "" };
readonly static char *end_comment[] =	{ "*/", "C", ";", "!", "}" };

/*
 Buffer for macroname from /MACRO=name qualifier and a separate buffer
 for the "facility" name from the macro (exclusive of "$" or "_" prefix
 and "DEF" or "_DEF" suffix).
*/
static char macroname[NAME_BUF], macrofac[NAME_LEN];
static $DESCRIPTOR( d_macroname, macroname);

/**********************************************
 * Global Procedure and Function definitions: *
 **********************************************/
status SYS$FILESCAN();			/* Scan file specification service */

status LIB$GET_FOREIGN();		/* Get foreign command line */
status LIB$GET_INPUT();			/* Get input from SYS$INPUT */
status CLI$DCL_PARSE();			/* Invoke DCL command parser */
status CLI$PRESENT();			/* Test for command component present */
status CLI$GET_VALUE();			/* Get command component value */
void LIB$STOP(), LIB$SIGNAL();		/* Stop on or signal condition */

address malloc();			/* Memory allocation function */
void free();				/* Memory deallocation routine */
int read();				/* Read a record */
void perror(), exit();			/* Error message and exit functions */
int strlen();				/* Length of NUL-terminated string */
int strspn();				/* Span over characters in set */


/* XDFO
 Extract definitions from an object code file.  Format is:

	$ XDFO [/qualifiers]  object-file

 Defined as a foreign DCL command.  See XDFO.HLP for more information.
*/
main()
{
register status sts;
register union {
    struct OBJ *objr;		/* Pointer to general object record */
    struct MHD *mhdr;		/* Pointer to module header record */
    struct GTX *gtxr;		/* Pointer to general text record */
    struct GSD *gsdr;		/* Pointer to Global Symbol Directory */
    } or;				/* Pointers to records buffer */
unsigned osize;				/* Size of object record buffer */
unsigned rsize;				/* Size of current object record */
register char *tp;			/* General text pointers */
unsigned ln;				/* General counters/lengths */
int process_gsd_entry();		/* Process GSD sub-records */
void setup_name_filter();		/* Setup name filter linked list */
char *dnap = NULL;			/* Default output file type */
register int commentblock = FALSE;	/* Flags if outputting comments */
char *strchr();				/* Locate character in string */
int strcmp();				/* String comparison routine */

/*
 Adjust foreign command line buffer descriptor for verb text.
*/
d_ftext.dsc$w_length -= VERB_LEN;
d_ftext.dsc$a_pointer += VERB_LEN;

sts = LIB$GET_FOREIGN( &d_ftext, 0, &d_ftext.dsc$w_length);
ABORT_ON_FAILURE( sts);

/*
 Readjust descriptor for XDFO command line.
*/
d_ftext.dsc$w_length += VERB_LEN;
d_ftext.dsc$a_pointer = ftext;

/*
 Decode the "DCL" command line for the XDFO command.
*/
sts = CLI$DCL_PARSE( &d_ftext, &XDFO_TABLES, LIB$GET_INPUT);
if ($VMS_FAILURE( sts))
    LIB$STOP( sts | STS$M_INHIB_MSG);

/*
 Check for the filter conditioning qualifiers and setup global variables.
*/
sts = CLI$PRESENT( &d_allhex);
if ($VMS_SUCCESS( sts))
    all_hex = TRUE;			/* /ALLHEX seen */
else if (sts != CLI$_ABSENT)
    all_hex = FALSE;			/* /NOALLHEX seen */

sts = CLI$PRESENT( &d_absolute);
if ($VMS_SUCCESS( sts))
    use_absolute = TRUE;		/* /ABSOLUTE seen */
else if (sts != CLI$_ABSENT)
    use_absolute = FALSE;		/* /NOABSOLUTE seen */

sts = CLI$PRESENT( &d_relocatable);
if ($VMS_SUCCESS( sts))
    use_relocatable = TRUE;		/* /RELOCATABLE seen */
else if (sts != CLI$_ABSENT)
    use_relocatable = FALSE;		/* /NORELOCATABLE seen */

sts = CLI$PRESENT( &d_weak);
if ($VMS_SUCCESS( sts))
    use_weak = TRUE;			/* /WEAK seen */
else if (sts != CLI$_ABSENT)
    use_weak = FALSE;			/* /NOWEAK seen */

/*
 Look for /INCLUDE or /EXCLUDE qualifiers and setup their respective linked
 lists of symbol names (w/wildcards) to be INCLUDEd or EXCLUDEd from the
 definitions file.
*/
sts = CLI$PRESENT( &d_include);
if ($VMS_SUCCESS( sts))
    setup_name_filter( &d_include, &includes);

sts = CLI$PRESENT( &d_exclude);
if ($VMS_SUCCESS( sts))
    setup_name_filter( &d_exclude, &excludes);

/*
 Setup definitions type selector and default output file type based on
 type qualifiers.
*/
if ($VMS_SUCCESS( CLI$PRESENT( &d_cc)))
    {
    deftype = CC;
    dnap = dna_cc;
    }

if ($VMS_SUCCESS( CLI$PRESENT( &d_fortran)))
    {
    deftype = FOR;
    dnap = dna_fortran;
    }

if ($VMS_SUCCESS( CLI$PRESENT( &d_macro)))
    {
    deftype = MAC;
    dnap = dna_macro;
    }

if ($VMS_SUCCESS( CLI$PRESENT( &d_options)))
    {
    deftype = OPT;
    dnap = dna_options;
    }

if ($VMS_SUCCESS( CLI$PRESENT( &d_pascal)))
    {
    deftype = PAS;
    dnap = dna_pascal;
    }

if (dnap ==NULL)			/* If no type selected ... */
    LIB$STOP( XDFO__NODEFTYP);

/*
 Get input file specification from P1 parameter and open the input file.
 Input is normally a .OBJ file.

 Then get actual input file specification after open and parse it.
*/
(void) CLI$GET_VALUE( &d_p1, &d_objfile, &d_objfile.dsc$w_length);
objfile[d_objfile.dsc$w_length] = '\0';	/* Supply NUL terminator */

obj = fopen( objfile, "r", "dna=.OBJ");
if (obj == NULL)
    LIB$STOP( XDFO__INPFILERR, 0, vaxc$errno);

(void) fgetname( obj, objfile);
d_objfile.dsc$w_length = strlen( objfile);

sts= SYS$FILESCAN( &d_objfile, &fileparts, 0);
ABORT_ON_FAILURE( sts);

/*
 Open output file.  Default is to use just the input file name with a default
 file type specified by the definitions type selection.  The output file will
 be a normal VAX text file.

 If the /OUTPUT qualifier is present, get the file specification from its
 value.
*/
if ($VMS_SUCCESS( CLI$PRESENT( &d_output)))
    {
    (void) CLI$GET_VALUE( &d_output, &d_deffile, &d_deffile.dsc$w_length);
    deffile[d_deffile.dsc$w_length] = '\0';
    }
else
    strncpy( deffile, fileparts.filename.caddr, fileparts.filename.clen);

defs = fopen( deffile, "w", dnap, "rfm=var", "rat=cr");
if (defs == NULL)
    LIB$STOP( XDFO__OUTFILERR, 0, vaxc$errno);

/*
 Get actual output file specification and parse it for later use.
*/
(void) fgetname( defs, deffile);
d_deffile.dsc$w_length = strlen( deffile);

sts= SYS$FILESCAN( &d_deffile, &fileparts, 0);
ABORT_ON_FAILURE( sts);

*(fileparts.filetype.caddr + fileparts.filetype.clen) = '\0';

/*
 If /MACRO was selected as the definitions type, then get the macro
 name from the qualifier value and check it for validity.
*/
if (deftype == MAC)
    {
    (void) CLI$GET_VALUE( &d_macro, &d_macroname, &d_macroname.dsc$w_length);
    macroname[d_macroname.dsc$w_length] = '\0';
    if ((macroname[0] == '$') || (macroname[0] == '_'))
					/* Copy name w/o prefix characters */
	strcpy( macrofac, macroname + strspn( macroname, "$_"));
    else
	strcpy( macrofac, macroname);	/* Copy complete name */

    tp = strchr( macrofac, '\0');	/* Find end of macro name */
    if (strcmp( tp-3, "DEF") == 0)
	*(tp-3) = '\0';			/* Lop off "DEF" at end */
    else if (strcmp( tp-4, "_DEF") == 0)
	*(tp-4) = '\0';			/* Lop off "_DEF" at end */
    else if (strcmp( tp-4, "$DEF") == 0)
	*(tp-4) = '\0';			/* Lop off "$DEF" at end */
    else
	LIB$SIGNAL( XDFO__MACRONAM);	/* No "DEF" in macro name */
    }

/*
 Get buffer for Object records.
*/
osize = OBJ$C_MAXRECSIZ;		/* Assume maximum size */
or.objr = (struct OBJ *) malloc( osize);	/* Get initial buffer */

/*
 Read the Object records
*/
while ((rsize = read( fileno(obj), or.objr, osize)) > 0)
    switch (or.objr->obj$b_rectyp)
	{
	case OBJ$C_HDR:			/* Header record */
	    switch (or.mhdr->mhd$b_hdrtyp)
		{
		case MHD$C_MHD:		/* Main Module Header */
		    commentblock = TRUE;
		    fprintf( defs, "%s\n", begin_comment[deftype]);
		    fprintf( defs, "%s %s\n", cont_comment[deftype],
			    fileparts.filename.caddr);
		    fprintf( defs, "%s\n", cont_comment[deftype]);
		    tp = or.mhdr->mhd$t_name + or.mhdr->mhd$b_namlng;
		    fprintf( defs, "%s From object module %.*s\n",
			    cont_comment[deftype],
			    or.mhdr->mhd$b_namlng, or.mhdr->mhd$t_name);
		    ln = (unsigned char) *tp++;	    /* Length of version text */
		    fprintf( defs, "%s Module version %.*s\n",
			    cont_comment[deftype], ln, tp);
		    tp = tp + ln;	    /* Advance over version */
		    fprintf( defs, "%s Compiled on %.17s\n",
			    cont_comment[deftype], tp);

		    if (or.mhdr->mhd$w_recsiz > osize)
			{
			/*
			 Release current OBJ buffer as possibly not large
			 enough for remaining records in file.
			*/
			free( or.objr);

			/*
			 Get new buffer for new maximum record size.
			*/
			osize = or.mhdr->mhd$w_recsiz;
			or.objr = (struct OBJ *) malloc( osize);
			}
		    break;

		case MHD$C_LNM:		/* Language Processor Name Header */
		    if (!commentblock)
			{
			commentblock = TRUE;
			fprintf( defs, "%s\n", begin_comment[deftype]);
			}
		    ln = rsize - 2*sizeof (unsigned char);
		    fprintf( defs, "%s Compiled by %.*s\n",
			    cont_comment[deftype], ln, or.gtxr->gtx$t_text);
		    break;

		case MHD$C_SRC:		/* Source Files Header */
		    if (!commentblock)
			{
			commentblock = TRUE;
			fprintf( defs, "%s\n", begin_comment[deftype]);
			}
		    ln = rsize - 2*sizeof (unsigned char);
		    fprintf( defs, "%s From source file %.*s\n",
			    cont_comment[deftype], ln, or.gtxr->gtx$t_text);
		    break;

		default:		/* Ignore all other Header records */
		    break;
		}
	    break;

	case OBJ$C_GSD:			/* Global Symbol Directory */
	    /*
	     End any partial comment block from Module Headers...
	    */
	    if (commentblock)
		{
		commentblock = FALSE;
		fprintf( defs, "%s\n", end_comment[deftype]);

		if (deftype == PAS)
		    fprintf( defs, "\nCONST\n");

		if (deftype == MAC)
		    {
		    fprintf( defs, "\n\t.MACRO  %s,$gbl\n", macroname);
		    fprintf( defs, "\t$DEFINI %s,$gbl\n\n", macrofac);
		    }
		}

	    /*
	     Get start of first sub-record in GSD record.
	    */
	    tp = (char *) or.gsdr->gsd$t_start;

	    /*
	     Loop over all GSD sub-records, processing each one and returning
	     its length.
	    */
	    while (tp < ((char *) or.gsdr + rsize))
		tp += process_gsd_entry( tp);
	    break;

	case OBJ$C_EOM:			/* End Of Module */
	case OBJ$C_EOMW:
	    /*
	     End any partial comment block left from Module Headers...
	    */
	    if (commentblock)
		{
		commentblock = FALSE;
		fprintf( defs, "%s\n", end_comment[deftype]);
		}

	    if (deftype == MAC)
		{
		fprintf( defs, "\n\t$DEFEND %s,$gbl,DEF\n", macrofac);
		fprintf( defs, "\t.ENDM %s\n", macroname);
		}

	default:			/* Ignore all other record types */
	    break;
	}

/*
 Input file done (EOF or error).  Close both files and exit.
*/
if (fclose( obj) < 0)
    LIB$STOP( XDFO__INPFILERR, 0, vaxc$errno);

if (fclose( defs) < 0)
    LIB$STOP( XDFO__OUTFILERR, 0, vaxc$errno);
}

/* Output_Definition
 File the symbol definitions for output and make output definition in
 correct format.
*/

void output_definition( gsyp, length, namep, value)
struct GSY *gsyp;			/* Pointer to generic GSD record */
unsigned char length;			/* Symbol name length */
char *namep;				/* Pointer to symbol name */
unsigned long value;			/* Symbol's value */
{
$DESCRIPTOR0( d_name);			/* Descriptor for symbol name */
register struct EIname *ein;		/* Include/Exclude descriptor ptr */ 
register status sts;
status STR$MATCH_WILD();		/* Match to pattern (w/wildcards) */
int STR$FIND_FIRST_SUBSTRING();		/* Search w/set of substrings */
int STR$POSITION();			/* Search for substring */
register int use_hex = all_hex;		/* Value output format flag */
int index, ssi;
RO$DESCRIPTOR( d_mask1, "$M_");		/* In name for bit mask symbol */
RO$DESCRIPTOR( d_mask2, "_M_");		/* In name for bit mask symbol */
RO$DESCRIPTOR( d_cond1, "$_");		/* In name for condition symbol */
RO$DESCRIPTOR( d_cond2, "__");		/* In name for condition symbol */
RO$DESCRIPTOR( d_notcond, "___");	/* Not name for condition symbol */

/*
 Now, check for WEAK symbol and not including such in output.
 Same for ABSOLUTE and RELOCATABLE symbols ...
*/
if (gsyp->gsy$v_weak && !use_weak)
    return;

if (gsyp->gsy$v_rel && !use_relocatable)
    return;

if (!gsyp->gsy$v_rel && !use_absolute)
    return;

/*
 Now setup string descriptor for name and test against any INCLUDEd or
 EXCLUDEd names.
*/
d_name.dsc$w_length = length;
d_name.dsc$a_pointer = namep;

for (ein = includes.first; ein != NULL; ein = ein->next)
    {
    sts = STR$MATCH_WILD( &d_name, &ein->desc);
    if ($VMS_SUCCESS( sts))
	break;				/* Found an INCLUDE-able name! */
    }
if ((includes.first != NULL) && (ein == NULL))
    return;				/* INCLUDEs list but no matches! */

for (ein = excludes.first; ein != NULL; ein = ein->next)
    {
    sts = STR$MATCH_WILD( &d_name, &ein->desc);
    if ($VMS_SUCCESS( sts))
	return;				/* This name is on EXCLUDE list */
    }

/*
 Now setup output format flag if not already set as always hexadecimal.
 Selection is based on symbol name.  If it contains "$M_" or "_M_", then
 the name is assumed to be for a bit mask and value is output in hex.
 If the name contains "$_" or "__" (but not "___"), then it is assumed
 to be for a condition code and value is also output in hex.  Otherwise,
 values are output in decimal.
*/
if (!use_hex)
    if (STR$FIND_FIRST_SUBSTRING( &d_name, &index, &ssi, &d_cond2, &d_cond1,
				    &d_mask2, &d_mask1))
	if (ssi == 1)
	    {
	    if (STR$POSITION( &d_name, &d_notcond, &index) == 0)
		use_hex = TRUE;
	    }
	else
	    use_hex = TRUE;

switch (deftype)
    {
    case CC:
	if (use_hex)
	    fprintf( defs, "#define %.*s\t0X%X\n", length, namep, value);
	else
	    fprintf( defs, "#define %.*s\t%d\n", length, namep, value);
	break;

    case FOR:
	if (use_hex)
	    fprintf( defs, "\tPARAMETER %.*s = '%X'X\n", length, namep, value);
	else
	    fprintf( defs, "\tPARAMETER %.*s = %d\n", length, namep, value);
	break;

    case MAC:
	if (use_hex)
	    fprintf( defs, "$EQU\t%.*s\t<^X%X>\n", length, namep, value);
	else
	    fprintf( defs, "$EQU\t%.*s\t%d\n", length, namep, value);
	break;

    case OPT:
	fprintf( defs, "UNIVERSAL=%.*s\n", length, namep);
	break;

    case PAS:
	if (use_hex)
	    fprintf( defs, "    %.*s = %%X'%X';\n", length, namep, value);
	else
	    fprintf( defs, "    %.*s = %d;\n", length, namep, value);
	break;
    }
}

/* Process_GSD_Entry
 Process Global Symbol Directory sub-record and return the sub-record's length.
*/

int process_gsd_entry( ep)
register union {		/* Pointer to ... */
    address any;			/* Generic pointer */
    struct GSY *gsy;			/* Generic symbol def./ref. */
    struct GPS *psc;			/* P-Sect definition */
    struct SGPS *spsc;			/* Shareable image P-Sect def. */
    struct SRF *srf;			/* Symbol reference */
    struct SDF *sdf;			/* Symbol definition */
    struct SDFV *sdfv;			/* Vectored symbol definition */
    struct SDFW *sdfw;			/* Symbol def. with P-Sect Word */
    struct EPM *epm;			/* Entry point definition */
    struct EPMV *epmv;			/* Vectored entry point def. */
    struct EPMW *epmw;			/* Entry point def. w/P-Sect Word */
    struct PRO *pro;			/* Procedure definition */
    struct PROV *prov;			/* Vectored procedure definition */
    struct PROW *prow;			/* Procedure def. w/P-Sect Word */
    struct IDC *idc;			/* Ident consistency check */
    struct ENV *env;			/* Environment def./ref. */
    struct LSY *lsy;			/* Generic local symbol def./ref. */
    struct LSRF *lsrf;			/* Local symbol reference */
    struct LSDF *lsdf;			/* Local symbol definition */
    struct LEPM *lepm;			/* Local entry point definition */
    struct LPRO *lpro;			/* Local procedure definition */
    } ep;
{
int process_pro_entry();		/* Process PRO argument descriptions */

switch (ep.gsy->gsy$b_gsdtyp)
    {
    case GSD$C_ENV:		/* Define environment */
	return (ep.env->env$b_namlng + OFFSET_OF( ENV, env$t_name));

    case GSD$C_EPM:		/* Entry point and mask definition */
	return (ep.epm->epm$b_namlng + OFFSET_OF( EPM, epm$t_name));

    case GSD$C_EPMV:		/* Vectored verion of EPM */
	return (ep.epmv->epmv$b_namlng + OFFSET_OF( EPMV, epmv$t_name));

    case GSD$C_EPMW:		/* Entry point mask with word psect */
	return (ep.epmw->epmw$b_namlng + OFFSET_OF( EPMW, epmw$t_name));

    case GSD$C_IDC:		/* Random entity check */
	return (ep.idc->idc$b_namlng + OFFSET_OF( IDC, idc$t_name));

    case GSD$C_LEPM:		/* Local symbol entry point definition */
	return (ep.lepm->lepm$b_namlng + OFFSET_OF( LEPM, lepm$t_name));

    case GSD$C_LPRO:		/* Local symbol procedure definition */
	return (ep.lpro->lpro$b_namlng + OFFSET_OF( LPRO, lpro$t_name) +
		process_pro_entry( &ep.lpro->lpro$t_name +
					ep.lpro->lpro$b_namlng));

    case GSD$C_LSY:		/* Local symbol */
	if (ep.lsy->lsy$v_def)
	    return (ep.lsdf->lsdf$b_namlng + OFFSET_OF( LSDF, lsdf$t_name));
	else
	    return (ep.lsrf->lsrf$b_namlng + OFFSET_OF( LSRF, lsrf$t_name));

    case GSD$C_PRO:		/* Procedure with formal arguments */
	return (ep.pro->pro$b_namlng + OFFSET_OF( PRO, pro$t_name) +
		process_pro_entry( &ep.pro->pro$t_name + ep.pro->pro$b_namlng));

    case GSD$C_PROV:		/* Vectored verion of PRO */
	return (ep.prov->prov$b_namlng + OFFSET_OF( PROV, prov$t_name) +
		process_pro_entry( &ep.prov->prov$t_name +
				    ep.prov->prov$b_namlng));

    case GSD$C_PROW:		/* Procedure with word psect */
	return (ep.prow->prow$b_namlng + OFFSET_OF( PROW, prow$t_name) +
		process_pro_entry( &ep.prow->prow$t_name +
				    ep.prow->prow$b_namlng));

    case GSD$C_PSC:		/* Psect definition */
	return (ep.psc->gps$b_namlng + OFFSET_OF( GPS, gps$t_name));

    case GSD$C_SPSC:		/* Shareable image psect definition */
	return (ep.spsc->sgps$b_namlng + OFFSET_OF( SGPS, sgps$t_name));

    case GSD$C_SYM:		/* Symbol specification */
	if (ep.gsy->gsy$v_def)
	    {
	    /*
	     Global symbol definition to be extracted to output...
	    */
	    output_definition( ep.sdf, ep.sdf->sdf$b_namlng,
				ep.sdf->sdf$t_name, ep.sdf->sdf$l_value);

	    return (ep.sdf->sdf$b_namlng + OFFSET_OF( SDF, sdf$t_name));
	    }
	else
	    return (ep.srf->srf$b_namlng + OFFSET_OF( SRF, srf$t_name));

    case GSD$C_SYMV:		/* Vectored (dual-valued) version of SYM */
	/*
	 Global symbol definition to be extracted to output...
	*/
	output_definition( ep.sdfv, ep.sdfv->sdfv$b_namlng,
			    ep.sdfv->sdfv$t_name, ep.sdfv->sdfv$l_value);

	return (ep.sdfv->sdfv$b_namlng + OFFSET_OF( SDFV, sdfv$t_name));

    case GSD$C_SYMW:		/* Symbol specification with word psect */
	/*
	 Global symbol definition to be extracted to output...
	*/
	output_definition( ep.sdfw, ep.sdfw->sdfw$b_namlng,
			    ep.sdfw->sdfw$t_name, ep.sdfw->sdfw$l_value);

	return (ep.sdfw->sdfw$b_namlng + OFFSET_OF( SDFW, sdfw$t_name));
    }
}

/* Process_PRO_Entry
 Process the FML and ARG records attached to the PROx sub-record.
*/

int process_pro_entry( ep)
register struct FML *ep;		/* Formal arguments description */
{
register struct ARG *ap;		/* Formal Argument Description */
register int ln = sizeof (struct FML);	/* Length counter */
register int n = ep->fml$b_minargs;	/* Argument count */

/*
 Get pointer to first argument description, continue loop until argument
 count is exhausted.
*/
for (ap = (struct ARG *)((address) ep + ln); n > 0; )
    {
    ln += sizeof (struct ARG) + ap->arg$b_bytecnt;
    ap = (struct ARG *)((address) ap + sizeof (struct ARG) + ap->arg$b_bytecnt);
    }

return (ln);
}

/* Setup_Name_Filter
 Setup linked list of symbol names which constitute a name filter (to include
 only or exclude some names from the definitions).
*/

void setup_name_filter( qualdp, lhp )
Descrp qualdp;				/* Pointer to qualifier name */
struct {
    struct EIname *first, *last;
    } *lhp;				/* Pointer to linked list head */
{
struct EIname *einp;			/* Pointer to linked list element */
readonly static struct dsc$descriptor dynamic0 =
    { 0, DSC$K_DTYPE_T, DSC$K_CLASS_D, NULL };	/* Bare dynamic string descr. */

/*
 Loop until end of name list (only last name will return SS$_NORMAL).
*/
do {
    /*
     Get filter list element and setup empty dynamic string descriptor.
    */
    einp = (struct EIname *) malloc( sizeof (struct EIname));
    einp->desc = dynamic0;
    einp->next = NULL;			/* Will be last element on list */

    if (lhp->last == NULL)
	lhp->first = lhp->last = einp;	/* First and only entry on list */
    else
	{
	lhp->last->next = einp;		/* Tack onto end of list */
	lhp->last = einp;		/* And move end-of-list pointer */
	}
    } while (CLI$GET_VALUE( qualdp, &einp->desc) != SS$_NORMAL);
}
