/*--- EDIT # 0112	29-MAY-1986 13:38   SYS$SYSROOT:[RVT.UTILS]LST.C;19 */
/*--- Previous edit 	29-MAY-1986 10:37   SYS$SYSROOT:[RVT.UTILS]LST.C;18 */
/*
.title lst
;+
; index	List files on stdout
;
; Usage
;	LST filename(s) [-n] [-tm] [-p] [-b] [-wz] [-d] [-x,y] [+a,c]
;
; in
;	
;
; out
;
;
; Description
;
;	Options are:
;
;		-n	don't print line numbers
;
;		-tm	list the last 'm' lines of the file
;
;		-p	pause every 20 lines if listing to a terminal
;
;		-b	print non-printable characters as \ooo,
;			where ooo is the octal value
;
;		-wz	line wrap with z characters per line,
;			default is 80
;
;		-d	write each filename to stderr
;
;		-x,y	list lines x to y inclusive
;			forms of this option are:
;				-x,y	lines x to y
;				-x,	lines x to EOF
;				-,y	lines 1 to y
;				-x	line x only
;
;		+a,c	list lines a-c to a+c inclusive
;			The default value for c is 2.
;
;	Wildcard filenames are accepted.
;	Automatic pagination occurs every 60 lines, if not listing
;	to a terminal.
;
;	^Z will terminate listing the current file.
;	^C will cause the program to exit.
;
;
; status
;	
;
; bugs
;	
;
; Updates
;	 Date		Vers 	Who	Description
;	 15-JUL-82	0001	AJK	Initial coding
;
;-
*/
#ifdef vax
#define ftty(a)	isatty(fileno(a))
#define flun(a)	fileno(a)
#define iovtoa(a,b)	fgetname(a,b)

/* Arm control C & Z traps */
armcz()
{}

/* Get file creation date & time */
crdate()
{}

#endif

#include	<stdio.h>
#include	<cx.h>

#define		MAX_UINT	65535	/* Max integer value */

#define		MAX_RANGES	10	/* Max # of range options */
#define		DEF_TAIL_SIZE	10	/* Default size of tail */
#define		DEF_OFFSET	2

#define		FILENAME_SIZE	256	/* Max # of chars in filename */
#define		MAX_RECORD_SIZE	200	/* Max # of bytes in record */

#define		LINES_CRT	20	/* # of lines to pause after */
#define		LINES_PAGE	60	/* # of lines on each page */

#define		SLEEP_TIME	1	/* # of seconds to pause */

#define		CTRLA		001
#define		CTRLC		003
#define		CTRLZ		032
#define		DEL		0177
#define		ESC		033
#define		CHAR_MASK	000377
/* CHAR_MASK is a mask to delete sign extension of characters > DEL */

#define		TAB_LENGTH	8	/* Max # of spaces for a tab */
#define		DEF_LINE_SIZE	80	/* Default line for line wrap */
#define		MIN_LINE_SIZE	20	/* Min line for line wrap */

unsigned int tail_flag;		/* # of lines in tail */
unsigned int numbers_flag;	/* flag for printing of line numbers */
unsigned int pause_flag;	/* flag for pausing during listing */
unsigned int wrap_flag;		/* # of characters per line */
unsigned int debug_flag;	/* flag to print filename to stderr */
unsigned int backslash_flag;	/* flag to print cntrl chars as \ooo */
unsigned int min_range[MAX_RANGES];	/* Min value of range option */
unsigned int max_range[MAX_RANGES];	/* Max value of range option */
unsigned int max_line;			/* MAX line of all ranges */
unsigned int line_count;		/* # of lines printed */
unsigned int page_number;		/* Current page of file */

FILE *file_ptr;			/* file struct for file being listed */

int inchar = 0;		/* Input char from terminal */
char crebuf[40]= {0};	/* Buffer for file creation date */

main(argc, argv)
int argc;
char *argv[];
{
   int i;
   int file_count;
   int range_index;	/* index of next range */
   char *cp;
   char c;
   char *tail_ptr;	/* Array of pointers for tail */
   char *assign_flag();
   unsigned int base;
   unsigned int offset;

   range_index = 0;
   file_count = 0;

#ifdef vms
   argc = redir(argc, argv);
#endif
#ifdef vax
   if (ftty(stdout))
      setbuf(stdout, malloc(BUFSIZ));
#endif

   /* Search all command line arguments for options */
   for (i = 1; i < argc; i++) {
      cp = argv[i];
      if ((c = *cp++) == '-') {
	 /* This is a possible option string preceded by '-' */
	 argv[i] = NULL;
	 while ((c = *cp++) != NULL) {
	    switch (tolower(c)) {
	    case 'n':
	       numbers_flag++;
	       break;

	    case 'p':
	       pause_flag++;
	       break;

	    case 'b':
	       backslash_flag++;
	       break;

	    case 'd':
	       debug_flag++;
	       break;

	    case 'w':
	       cp = assign_flag(cp, &wrap_flag);
	       break;

	    case 't':
	       cp = assign_flag(cp, &tail_flag);
	       if (tail_flag == 0)
		  tail_flag = DEF_TAIL_SIZE;
	       break;

	    default:
	       if (isdigit(c) || c == ',') {
		  /* This is a form of the range option */
		  if (range_index >= MAX_RANGES)
		     error("Too many ranges specified", NULL);

		  /* Assign the min and max line numbers of the range */
		  cp = assign_flag(--cp, &min_range[range_index]);
		  if (*cp == ',') {
		     /* It's a range - assign the MAX of the range */
		     cp = assign_flag(++cp, &max_range[range_index]);
		     if (max_range[range_index] == 0)
			max_range[range_index] = MAX_UINT;
		  }
		  else
		     /* It's one line - MAX = MIN */
		  max_range[range_index] = min_range[range_index];

		  /* Keep track of the maximum line of all ranges */
		  if (max_range[range_index] > max_line)
		     max_line = max_range[range_index];

		  range_index++;
	       }
	       else
		  usage(c);
	    }
	 }
      }
      else if (c == '+') {
	 /* This is a possible option string preceded by '+' */
	 argv[i] = NULL;
	 while ((c = *cp++) != NULL) {
	    switch (tolower(c)) {
	    default:
	       if (isdigit(c) || c == ',') {
		  /* This is a base/offset option */
		  if (range_index >= MAX_RANGES)
		     error("Too many ranges specified", NULL);

		  /* Assign base & offset to find min/max line #'s */
		  cp = assign_flag(--cp, &base);
		  if (*cp == ',') {
		     cp = assign_flag(++cp, &offset);
		     if (offset == 0)
			offset = DEF_OFFSET;
		  }
		  else
		     offset = DEF_OFFSET;
		  if ((min_range[range_index] = base - offset) >= base)
		     min_range[range_index] = 0;
		  if ((max_range[range_index] = base + offset) <= base)
		     max_range[range_index] = MAX_UINT;

		  /* Keep track of the maximum line of all ranges */
		  if (max_range[range_index] > max_line)
		     max_line = max_range[range_index];

		  range_index++;
	       }
	       else
		  usage(c);
	    }
	 }
      }
      else
	 file_count++;
   }

   /* Complete the range options */
   max_range[range_index] = min_range[range_index] = 0;
   if (max_line == 0)
      max_line = MAX_UINT;

   /* Make sure line size is large enough */
   if (wrap_flag < MIN_LINE_SIZE)
      wrap_flag = DEF_LINE_SIZE;

   if (file_count == 0)
      error("No file specified", NULL);

   if (tail_flag != 0) {
      /* Allocate the pointer array for the tail */
      if ((tail_ptr = calloc(tail_flag, sizeof(char *))) == NULL)
	 error("Can't allocate array for tail", NULL);
   }

   /* Arm ^C & ^Z only if listing to a terminal */
   if (ftty(stdout))
      armcz(flun(stdin));

   /* List all filespecs in the command line */
   for (i = 1; i < argc; argv[i++] = NULL) {
      if ((cp = argv[i]) != NULL) {
	 if (list_filespec(cp, tail_ptr) == NULL)
	    exit();
      }
   }
}





/**********************************************************/
/* ASSIGN_FLAG assigns a value to a numeric flag from the */
/* command line.					  */
/* It returns a pointer to the character after the flag.  */

char *assign_flag(cp, flag)
char *cp;
unsigned int *flag;
{
   *flag = atoi(cp);
   while (isdigit(*cp))
      cp++;
   return (cp);
}





/***********************************************************/
/* LIST_FILESPEC lists one filespec from the command line. */

list_filespec(filespec, tail_ptr)
char *filespec;
char *tail_ptr[];
{
   int file_count;

   if ((file_ptr = fwild(filespec, "r")) == NULL) {
      printf("\n%s is an incorrect filespec\n\n", filespec);
      return (IS_SUC);
   }

   /* List each file in the filespec */
   for (file_count = 0; (file_ptr = fnext(file_ptr)) != NULL; file_count++) {
      if (list_one_file(tail_ptr) == NULL) {
	 fclose(file_ptr);
	 return (NULL);
      }
   }
   fclose(file_ptr);

   if (file_count == 0) {
      printf("\nNo matching files for %s\n\n", filespec);
   }
   return (IS_SUC);
}





/***********************************************************/
/* LIST_ONE_FILE lists an individual file from a filespec. */

list_one_file(tail_ptr)
char *tail_ptr[];
{
   int rec_length;	       /* length of record read from file */
   int line_number;
   int tail_index;	       /* index of next entry in tail_ptr[] */
   int tail_line;	       /* line # of first entry in tail_ptr[] */
   char buffer[MAX_RECORD_SIZE + 1];
   char file_name[FILENAME_SIZE];
   char last_char;	       /* last character of the last line */

   tail_index = 0;
   tail_line = 1;
   line_count = 0;
   page_number = 1;
   line_number = 1;
   last_char = NULL;

   /* Get creation date of input file */
   if (!ftty(file_ptr)) {
      crdate(flun(file_ptr));
   }

   iovtoa(file_ptr, file_name);

   if (debug_flag != 0)
      fprintf(stderr, "\n%s\n", file_name);

   /* Print the lines from the file */
   while (fgetss(buffer, MAX_RECORD_SIZE, file_ptr) != NULL &&
	(tail_flag != 0 || line_number <= max_line)) {
      /* Check for line ending with ESC followed by a blank line */
      if (last_char != ESC || *buffer != NULL) {
	 if (tail_flag != 0) {
	    /* Do tail keeping */
	    if (tail_ptr[tail_index] != NULL) {
	       free(tail_ptr[tail_index]);
	       tail_line++;
	    }

	    /* Put the line in the tail */
	    if ((tail_ptr[tail_index] = calloc(strlen(buffer),
		 sizeof(char))) == NULL) {
	       printf("\nMemory is available for only %d tail lines\n",
		    tail_index);
	       return (NULL);
	    }
	    strcpy(tail_ptr[tail_index], buffer);

	    /* increment tail_index mod tail_flag */
	    if (++tail_index == tail_flag)
	       tail_index = 0;
	 }

	 /* Check for ^C & ^Z before printing line */
	 if (inchar == CTRLC)
	    return (NULL);
	 else if (inchar == CTRLZ) {
	    /* Clean up before returning */
	    inchar = NULL;
	    if (tail_flag != 0)
	       free_tail(tail_ptr, tail_index);
	    return (IS_SUC);
	 }

	 if (line_to_print(line_number))
	    print_line(line_number, buffer, file_name);
	 line_number++;
      }

      /* Get the last character from the line */
      if ((rec_length = strlen(buffer)) > 0)
	 last_char = buffer[rec_length - 1];
      else
	 last_char = NULL;
   }

   if (tail_flag != 0) {
      if (list_tail(tail_ptr, tail_index, tail_line, file_name) == NULL)
	 return (NULL);
      free_tail(tail_ptr, tail_index);
   }
   return (IS_SUC);
}





/*******************************************************/
/* DO_PAGING does the paging/pausing after the correct */
/* number of lines				       */

do_paging(file_name)
char file_name[];
{
   if (ftty(stdout)) {
      /* listing to a terminal - pause if requested */
      if (pause_flag != 0 && line_count % LINES_CRT == 0)
	 sleep(SLEEP_TIME);
      if (line_count == 0)
	 printf("\n\n%s\n\n", file_name);
   }
   else {
      /* listing to a file - do automatic pagination */
      if (line_count % LINES_PAGE == 0) {
	 printf("\f%s\t\t", file_name);
	 if (!ftty(file_ptr)) {
	    printf("%.2s-%.3s-%.2s  %.2s:%.2s:%.2s",
		 &crebuf[15], &crebuf[17], &crebuf[20],
		 &crebuf[22], &crebuf[24], &crebuf[26]);
	 }
	 else
	    printf("\t\t\t\t");
	 printf("\t\tPage %d\n\n", page_number++);
      }
   }
}





/*****************************************/
/* LIST_TAIL prints the tail of the file */

list_tail(tail_ptr, tail_index, tail_line, file_name)
char *tail_ptr[];
int tail_index;
int tail_line;
char file_name[];
{
   int i;
   int line_number;

   /* Find the start of the tail */
   if (tail_ptr[tail_index] != NULL)
      i = tail_index;
   else
      i = 0;

   for (line_number = tail_line; tail_ptr[i] != NULL &&
	line_number < tail_line + tail_flag; line_number++) {

      /* Check for ^C & ^Z before printing line */
      if (inchar == CTRLC)
	 return (NULL);
      else if (inchar == CTRLZ) {
	 /* Clean up before returning */
	 inchar = NULL;
	 return (IS_SUC);
      }

      print_line(line_number, tail_ptr[i], file_name);
      /* Increment i mod tail_flag */
      if (++i == tail_flag)
	 i = 0;
   }
   return (IS_SUC);
}





/***************************************************/
/* FREE_TAIL deallocates buffers used for the tail */

free_tail(tail_ptr, tail_index)
char *tail_ptr[];
int tail_index;
{
   int i;

   /* Find the start of the tail */
   if (tail_ptr[tail_index] != NULL)
      i = tail_index;
   else
      i = 0;

   while (tail_ptr[i] != NULL) {
      free(tail_ptr[i]);
      tail_ptr[i] = NULL;
      /* Increment i mod tail_flag */
      if (++i == tail_flag)
	 i = 0;
   }
}





/****************************************************/
/* LINE_TO_PRINT checks if a line is to be printed. */

line_to_print(line_number)
int line_number;
{
   int i;

   /* Compare line # with all ranges */
   for (i = 0; i < MAX_RANGES && max_range[i] != 0; i++) {
      if (line_number >= min_range[i] && line_number <= max_range[i])
	 return (TRUE);
   }

   if (i == 0 && tail_flag == 0)
      return (TRUE);
   else
      return (FALSE);
}





/**************************************************************/
/* PRINT_LINE prints the characters from a line to the stdout */
/* with line wrap.					      */

print_line(line_number, buffer, file_name)
int line_number;
char buffer[];
char file_name[];
{
   int column;			/* Column # for next character */
   int char_length;		/* # of characters for expanded char */
   char c, lc;
   char char_buffer[TAB_LENGTH];

   do_paging(file_name);
   if (numbers_flag == 0) {
      printf("%5d.\t", line_number);
      column = TAB_LENGTH;
   }
   else
      column = 0;

   lc=c=0;
   while (lc=c, (c = *buffer++) != NULL) {
      /* Expand the char & determine the new column number */
      convert_char(c, char_buffer);
      if (c == '\t')
	 char_length = TAB_LENGTH - column % TAB_LENGTH;
      else
	 char_length = strlen(char_buffer);

      if ((column += char_length) > wrap_flag) {
	 /* Do line wrap & operations to keep things running right */
	 putchar('\n');
	 line_count++;
	 do_paging(file_name);
	 column = char_length;
	 if (numbers_flag == 0) {
	    putchar('\t');
	    column += TAB_LENGTH;
	 }
      }

      printf("%s", char_buffer);
   }            

   putchar('\n');
   if (lc == '\f') {	/* Form-feed....force page eject */
      line_count = LINES_PAGE;
      do_paging(file_name);
   }
#ifdef vax
   if (ftty(stdout))
      fflush(stdout);
#endif
   line_count++;
}





/***********************************************************/
/* CONVERT_CHAR converts a character to a printable format */

convert_char(c, char_buffer)
char c;
char *char_buffer;
{
   /* Determine char type & fill buffer with appropriate characters */
   if (isascii(c)) {
      if (iscntrl(c) && c != '\t') {
	 if (c == ESC) {
	    *char_buffer++ = '$';
	    *char_buffer = NULL;
	 }
	 else {
	    if (backslash_flag == 0) {
	       *char_buffer++ = '^';
	       *char_buffer++ = c + 'A' - CTRLA;
	       *char_buffer = NULL;
	    }
	    else {
	       sprintf(char_buffer, "\\%03o", c);
	    }
	 }
      }
      else {
	 *char_buffer++ = c;
	 *char_buffer = NULL;
      }
   }
   else {
      sprintf(char_buffer, "\\%03o", c & CHAR_MASK);
   }
}




#ifndef iscntrl
/***********************************************************/
/* ISCNTRL checks if the character is a control character. */

iscntrl(c)
char c;
{
   if (c < ' ' || c == DEL)
      return (TRUE);
   else
      return (FALSE);
}
#endif




#ifndef isascii
/*********************************************************/
/* ISASCII checks if the character is in the ascii range */

isascii(c)
char c;
{
   if (c < DEL && c > 0)
      return (TRUE);
   else
      return (FALSE);
}
#endif





usage(c)
char c;
{
   printf("\nUsage: LST filename(s) [-n] [-tm] [-p] [-b]");
   printf(" [-wz] [-d] [-x,y] [+a,c]\n");
   if (c == '?') {
      printf("\t-n\tdon't print line numbers\n");
      printf("\t-tm\tlist the last 'm' lines of the file\n");
      printf("\t-p\tpause every 20 lines if listing to a terminal\n");
      printf("\t-b\tprint non-printable characters as \\ooo\n");
      printf("\t-wz\tline wrap with z characters per line\n");
      printf("\t-d\twrite each filename to stderr\n");
      printf("\t-x,y\tlist lines x to y inclusive\n");
      printf("\t+a,c\tlist lines a-c to a+c inclusive\n");
   }
   exit();
}
